#!/usr/bin/expect -f

###########################################################################
###     Copyright (c) 2006 Hewlett Packard Development Company, L.P.      #
###########################################################################
### Remote host control script

set host $env(HPSUM_HOST)
set user $env(HPSUM_USER)
set pwd $env(HPSUM_PWD)
set service_control $env(HPSUM_SERVICE_CONTROL)

set step 0
set timeout -1
set connected_flag 0
set force_conservative 1  ;# set to 1 to force conservative mode even if
						  ;# script wasn't run conservatively originally
if { $force_conservative } {
	set send_slow { 1 .1 }
	proc send_with_id { id arg } {
		set spawn_id $id
		sleep .1
		exp_send -s -- $arg
	}
	proc send {arg} {
		sleep .1
		exp_send -s -- $arg
	}
}

#
# Returns a list of interfaces addresses excluding localhost
#
proc gethostipaddress { } {
	puts "getting ip address\r"
	set ips [list]
	if { [ file executable /usr/sbin/ifconfig ] } {
		catch { ::exec /usr/sbin/ifconfig -a } data
	} elseif { [ file executable /sbin/ifconfig ] } {
		catch { ::exec /sbin/ifconfig -a } data
	}

	foreach line [split $data "\n" ] {
		regexp {^[a-z]+\d+} $line if
		if { [ regexp {^\s+inet\s+(?:addr:)?(\S+)} $line -> ip ] } {
			if { $ip != "127.0.0.1" } {
				puts "ip: $ip"
				lappend ips $ip
			}
		}
	}
	return $ips
}

proc copy_file { step file host user pwd destdir toremote } {

	set connected_flag 0

	if { $toremote } {
		spawn /usr/bin/scp -p $file $user@$host:$destdir
	} else {
		spawn /usr/bin/scp -p $user@$host:$destdir $file
	}

	expect {
		"assword:" { puts "($step.1) ControlLinuxServer: sending password\r"; send_with_id $spawn_id "$pwd\r"; set connected_flag 1; exp_continue }
		"No such file" { puts "($step.2) ControlLinuxServer: cannot copy file $file to $host:$destdir\r"; exit [expr $step * 10 + 4] }
		"denied" {  puts "($step.3) ControlLinuxServer: permision denided\r"; exit [expr $step * 10 + 3] }
		"Command not found" { puts "($step.4) ControlLinuxServer: command not found\r"; exit [expr $step * 10 + 5] }
		"not permitted" { puts "($step.5) ControlLinuxServer: Operation not permitted\r"; exit [expr $step * 10 + 6] }
		"Text file busy" { puts "($step.6) ControlLinuxServer: Unable to overwrite file $file\r"; exit [expr $step * 10 + 9] }
		"fingerprint" { puts "($step.7) ControlLinuxServer: sending yes\r"; send_with_id $spawn_id "yes\r"; exp_continue}
		timeout { puts "($step.8) ControlLinuxServer: timeout\r", exit 11 }
		eof {
			if {$connected_flag} { 
				puts "($step.9) ControlLinuxServer: $file copied\r"
			} else {
				puts "($step.10) ControlLinuxServer: no route to the host\r"; exit [expr $step * 10 + 2] };
		}
	}
}

#### Start SOAP server on the remote host
if { $service_control == "start" } {

set service_exe $env(HPSUM_SERVICE_EXE)
set service_args $env(HPSUM_SVCARG)
puts "$service_args\r"
set destdir $env(HPSUM_DESTDIR)
set skipHost $env(HPSUM_SKIPHOST)

puts "($step.0) ControlLinuxServer: Checking for running $service_exe on $host\r"

# Add entry to remote firewall
set lst		[ split $service_args " " ]
set ipaddr	[ lindex $lst 1 ]
set portno	[ lindex $lst 2 ]
set ips		[ gethostipaddress ]
foreach ip $ips {
	puts $ip
	set connected_flag 0

	# Add to the INPUT chain
	spawn /usr/bin/ssh $user@$host iptables -I INPUT 1 -p TCP -s $ip -j ACCEPT
	expect {
		"assword:" { puts "(0.1) sending password "; send "$pwd\r"; set connected_flag 1; exp_continue }
		"denied" {  puts "(${step}.2) iptables: permision denided\r"; exit ${step}3 }
		"Command not found" { puts "(${step}.3) iptables: command not found\r"; exit ${step}5 }
		"fingerprint" { puts "(${step}.4) sending yes"; send $spawn_id "yes\r"; exp_continue}
		timeout { puts "timeout\r", exit ${step}1 }
		eof {
			if {$connected_flag} { 
				puts "(${step}.5) iptables opened port in  firewall\r"
			} else {
				puts "(${step}.6) iptables: no route to the host\r"; exit ${step}2 };
		}
	}

	# Add to the OUTPUT chain
	spawn /usr/bin/ssh $user@$host iptables -I OUTPUT 1 -p TCP -d $ip -j ACCEPT
	expect {
		"assword:" { puts "(0.1) sending password "; send "$pwd\r"; set connected_flag 1; exp_continue }
		"denied" {  puts "(${step}.2) iptables: permision denided\r"; exit ${step}3 }
		"Command not found" { puts "(${step}.3) iptables: command not found\r"; exit ${step}5 }
		"fingerprint" { puts "(${step}.4) sending yes"; send $spawn_id "yes\r"; exp_continue}
		timeout { puts "timeout\r", exit ${step}1 }
		eof {
			if {$connected_flag} { 
				puts "(${step}.5) iptables opened port in  firewall\r"
			} else {
				puts "(${step}.6) iptables: no route to the host\r"; exit ${step}2 };
		}
	}
}

spawn /usr/bin/ssh $user@$host ps -ef | grep $service_exe | grep -v grep
set connected_flag 0
set found_running_service 0

expect {
	"assword:" { puts "($step.1) ControlLinuxServer: sending password\r"; send "$pwd\r"; set connected_flag 1; exp_continue }
	"Command not found" { puts "($step.2) ControlLinuxServer: command not found\r"; exit ${step}5 }
	"not permitted" { puts "(${step}.3) ControlLinuxServer: Operation not permitted\r"; exit ${step}6 }
	"denied" {  puts "(${step}.4) ControlLinuxServer: wrong username or password\r"; exit ${step}3}
	"fingerprint" { puts "(${step}.5) ControlLinuxServer: sending yes\r"; send "yes\r"; exp_continue}
	"$service_exe" { 
			puts "(${step}.6) ControlLinuxServer: found running copy of $service_exe on $host and flag is $skipHost\r";
			if { $skipHost == "1" } {
				puts "(${step}.7) ControlLinuxServer: host $host skipped\r";
				exit 99
			} else {
				set found_running_service 1
			};
	}	
	timeout { puts "(${step}.8) ControlLinuxServer: timeout\r", exit 01 }
	eof {
		if {$connected_flag} { 
			puts "(${step}.9) ControlLinuxServer: remote system is clean\r"
		} else {
			puts "(${step}.10) ControlLinuxServer: no route to the host\r"; exit ${step}2 };
	}
}

set step 1
if { $found_running_service } {
puts "(${step}.0) Killing running $service_exe and tools on $host\r"

spawn /usr/bin/ssh $user@$host killall -q $service_exe hpsum_discagnt hpsum_hwdiscovery InstallClient hpsum_srv_discovery
set connected_flag 0

expect {
	"assword:" { puts "(${step}.1) ControlLinuxServer: sending password\r"; send "$pwd\r"; set connected_flag 1; exp_continue }
	"Command not found" { puts "(${step}.2) ControlLinuxServer: command not found\r"; exit ${step}5 }
	"not permitted" { puts "(${step}.3) ControlLinuxServer: Operation not permitted\r"; exit ${step}6 }
	"no process killed" { exp_continue }
	"denied" {  puts "(${step}.4) ControlLinuxServer: wrong username or password\r"; exit ${step}3 }
	"fingerprint" { puts "(${step}.5) ControlLinuxServer: sending yes"; send "yes\r"; exp_continue }
	timeout { puts "(${step}.6) ControlLinuxServer: timeout\r", exit ${step}1 }
	eof {
		if {$connected_flag} { 
			puts "(${step}.7) ControlLinuxServer: killing done\r"
		} else {
			puts "(${step}.8) ControlLinuxServer: no route to the host\r"; exit ${step}2 };
    	}
}
}

set step 2
puts "(${step}.0)Creating /tmp/hp_sum\r"
set connected_flag 0

spawn /usr/bin/ssh $user@$host /bin/mkdir $destdir

expect {
	"assword:" { puts "(${step}.1) ControlLinuxServer: sending password\r"; send "$pwd\r"; set connected_flag 1; exp_continue }
	"exists" {  puts "(${step}.2) ControlLinuxServer: $destdir directory exists\r"; exp_continue}
	"cannot" {  puts "(${step}.3) ControlLinuxServer: cannot create directory $destdir\r"; exit ${step}4}
	"not a directory" {  puts "(2.4) ControlLinuxServer: file $destdir exists - cannot create directory $destdir\r"; exit ${step}7}
	"denied" {  puts "(${step}.5) ControlLinuxServer: wrong username or password\r"; exit ${step}3}
	"Command not found" { puts "(${step}.6) ControlLinuxServer: command not found\r"; exit ${step}5 }
	"not permitted" { puts "(${step}.7) ControlLinuxServer: Operation not permitted\r"; exit ${step}6 }
	"fingerprint" { puts "(${step}.8) ControlLinuxServer: sending yes\r"; send "yes\r"; exp_continue}
	timeout { puts "(${step}.9) ControlLinuxServer: timeout\r", exit ${step}1 }
	eof {
		if {$connected_flag} { 
			puts "(${step}.10) ControlLinuxServer: mkdir done\r"
		} else {
			puts "(${step}.11) ControlLinuxServer: no route to the host\r"; exit ${step}2 };
	}
}

set step 5
puts "(${step}.0) ControlLinuxServer: copying $service_exe to $host\r"
copy_file 3 $service_exe $host $user $pwd $destdir 1

puts "(${step}.0) ControlLinuxServer: ssh for $user@$host $destdir $service_exe $service_args\r"
set connected_flag 0

spawn /usr/bin/ssh $user@$host $destdir/$service_exe $service_args&
expect {
	"assword:" { puts "(${step}.1) ControlLinuxServer: sending password\r"; send "$pwd\r"; set connected_flag 1; exp_continue }
	"denied" {  puts "(${step}.2) ControlLinuxServer: permision denided\r"; exit ${step}3 }
	"Command not found" { puts "(${step}.3) ControlLinuxServer: command not found\r"; exit ${step}5 }
	"not permitted" { puts "(${step}.4) ControlLinuxServer: Operation not permitted\r"; exit ${step}6 }
	"fingerprint" { puts "(${step}.5) ControlLinuxServer: sending yes\r"; send "yes\r"; exp_continue}
	"HPServer" { puts "(${step}.6) Server started\r"; exit 0}
	timeout { puts "(${step}.7) ControlLinuxServer: timeout\r", exit ${step}1 }
	eof {
		if {$connected_flag} { 
			puts "(${step}.8) ControlLinuxServer: service created\r"
		} else {
			puts "(${step}.9) ControlLinuxServer: no route to the host\r"; exit ${step}2 };
	}
}
}

#### Stop SOAP server and tools on the remote host
set step 6
if { $service_control == "stop" } {

set service_exe $env(HPSUM_SERVICE_EXE)
set destdir $env(HPSUM_DESTDIR)

	puts "(${step}.0) ControlLinuxServer: Killing server\r"
	set connected_flag 0
	
	spawn /usr/bin/ssh $user@$host killall $service_exe hpsum_discagnt hpsum_hwdiscovery hpsum_srvdiscovery InstallClient hpsum_srv_discovery

	expect {
		"assword:" { puts "(${step}.1) ControlLinuxServer: sending password\r"; send "$pwd\r"; set connected_flag 1; exp_continue }
		"denied" {  puts "(${step}.2) ControlLinuxServer: permision denided\r"; exit ${step}3 }
		"Command not found" { puts "(${step}.3) ControlLinuxServer: command not found\r"; exit ${step}5 }
		"not permitted" { puts "(${step}.4) ControlLinuxServer: Operation not permitted\r"; exit ${step}6 }
		"no process killed" { exp_continue }
		"fingerprint" { puts "(${step}.5) ControlLinuxServer: sending yes\r"; send "yes\r"; exp_continue}
		timeout { puts "(${step}.6) ControlLinuxServer: timeout\r", exit ${step}1 }
		eof {
			if {$connected_flag} { 
				puts "(${step}.7) ControlLinuxServer: service started\r"
			} else {
				puts "(${step}.8) ControlLinuxServer: no route to the host\r"; exit ${step}2 };
		}
	}

	set step 7
	puts "(${step}.0) ControlLinuxServer: Cleaning temporary directory\r"
	set connected_flag 0
	
	spawn /usr/bin/ssh $user@$host /bin/rm -r -f $destdir

	expect {
		"assword:" { puts "(${step}.1) ControlLinuxServer: sending password\r"; send "$pwd\r"; set connected_flag 1; exp_continue }
		"denied" {  puts "(${step}.2) ControlLinuxServer: permision denided\r"; exit ${step}3 }
		"Command not found" { puts "(${step}.3) ControlLinuxServer: command not found\r"; exit ${step}5 }
		"not permitted" { puts "(${step}.4) ControlLinuxServer: Operation not permitted\r"; exit ${step}6 }
		"fingerprint" { puts "(${step}.5) ControlLinuxServer: sending yes\r"; send "yes\r"; exp_continue}
		timeout { puts "(${step}.6) ControlLinuxServer: timeout\r", exit ${step}1 }
		eof {
			if {$connected_flag} { 
				puts "(${step}.7) ControlLinuxServer: /tmp/hp_sum deleted\r"
			} else {
				puts "(${step}.8) ControlLinuxServer: no route to the host\r"; exit ${step}2 };
		}
	}

	# Remove entry from remote firewall
	set portno	4242
	set ips		[ gethostipaddress ]
	foreach ip $ips {
		puts $ip
		puts "$step: removeentryfromfirewall\r"

		# Remove from the INPUT chain
		spawn /usr/bin/ssh $user@$host iptables -D INPUT -p TCP -s $ip -j ACCEPT
		expect {
			"assword:" { puts "($step.1) sending pwd "; send "$pwd\r"; set connected_flag 1; exp_continue }
			"denied" {  puts "($step.2) iptables: permision denided\r"; exit ${step}2 }
			"Command not found" { puts "($step.3) iptables: command not found\r"; exit ${step}3 }
			"fingerprint" { puts "($step.4) sending yes"; send $spawn_id "yes\r"; exp_continue}
			timeout { puts "timeout\r", exit ${step}1 }
			eof {
				if {$connected_flag} { 
					puts "($step.5) iptables opened port in  firewall\r"
				} else {
					puts "($step.6) iptables: no route to the host\r"; exit ${step}6 };
			}
		}

		# Remove from the OUTPUT chain
		spawn /usr/bin/ssh $user@$host iptables -D OUTPUT -p TCP -d $ip -j ACCEPT
		expect {
			"assword:" { puts "($step.1) sending pwd "; send "$pwd\r"; set connected_flag 1; exp_continue }
			"denied" {  puts "($step.2) iptables: permision denided\r"; exit ${step}2 }
			"Command not found" { puts "($step.3) iptables: command not found\r"; exit ${step}3 }
			"fingerprint" { puts "($step.4) sending yes"; send $spawn_id "yes\r"; exp_continue}
			timeout { puts "timeout\r", exit ${step}1 }
			eof {
				if {$connected_flag} { 
					puts "($step.5) iptables opened port in  firewall\r"
				} else {
					puts "($step.6) iptables: no route to the host\r"; exit ${step}6 };
			}
		}
	}
}

#### Check if a remote host supports ssh 
set step 8
if { $service_control == "check" } {

puts "(${step}.0) ControlLinuxServer: Connecting to host $host\r"
spawn /usr/bin/ssh $user@$host pwd
set connected_flag 0

expect {
	"assword:" { puts "(${step}.1) ControlLinuxServer: sending password\r"; send "$pwd\r"; set connected_flag 1; exp_continue }
	"Command not found" { puts "(${step}.2) ControlLinuxServer: command not found\r"; exit ${step}5 }
	"not permitted" { puts "(${step}.3) ControlLinuxServer: Operation not permitted\r"; exit ${step}6 }
	"denied" {  puts "(${step}.4) ControlLinuxServer: wrong username or password\r"; exit ${step}3}
	"fingerprint" { puts "(${step}.5) ControlLinuxServer: sending yes"; send "yes\r"; exp_continue}
	timeout { puts "(${step}.6) ControlLinuxServer: timeout\r", exit ${step}1 }
	eof {
		if {$connected_flag} { 
			puts "(${step}.7) ControlLinuxServer: connected succesfully\r"
		} else {
			puts "(${step}.8) ControlLinuxServer: no route to the host\r"; exit ${step}2 };
	}
}
}

#### Reboot remote host 
set step 9
if { $service_control == "reboot" } {

set rebootargs $env(HPSUM_REBOOTARGS)

puts "(${step}.0) ControlLinuxServer: Rebooting host $host\r"
spawn /usr/bin/ssh $user@$host /sbin/shutdown $rebootargs
set connected_flag 0

expect {
	"assword:" { puts "(${step}.1) ControlLinuxServer: sending password\r"; send "$pwd\r"; set connected_flag 1; exp_continue }
	"Command not found" { puts "(${step}.2) ControlLinuxServer: command not found\r"; exit ${step}5 }
	"not permitted" { puts "(${step}.3) ControlLinuxServer: Operation not permitted\r"; exit ${step}6 }
	"denied" {  puts "(${step}.4) ControlLinuxServer: wrong username or password\r"; exit ${step}3}
	"fingerprint" { puts "(${step}.5) ControlLinuxServer: sending yes"; send "yes\r"; exp_continue}
	timeout { puts "(${step}.6) ControlLinuxServer: timeout\r", exit ${step}1 }
	eof {
		if {$connected_flag} { 
			puts "(${step}.7) ControlLinuxServer: Host $host rebooted succesfully\r"
		} else {
			puts "(${step}.8) ControlLinuxServer: no route to the host\r"; exit ${step}2 };
	}
}
}
